/*	animcanvas.cc
	Animation canvas functions
	Copyright (C) 2005 Lawrence Sebald

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "animdialog.h"
#include "animcanvas.h"

ccAnimCanvas::ccAnimCanvas(wxWindow *parent, ccMap *m, ccAnim *a) :
wxScrolledWindow(parent, -1, wxDefaultPosition, wxSize(200, 
(int)(m->getTileHeight() * 1.5f)))	{
	map = m;
	tile = 0;
	anim = a;

	SetVirtualSize(m->getTileWidth() * a->getTileCount(), m->getTileHeight());
	SetScrollRate(m->getTileWidth(), m->getTileHeight());
}

ccAnimCanvas::~ccAnimCanvas()	{
}

BEGIN_EVENT_TABLE(ccAnimCanvas, wxScrolledWindow)
	EVT_LEFT_DOWN(ccAnimCanvas::onLeftDown)
END_EVENT_TABLE()

void ccAnimCanvas::OnDraw(wxDC &context)	{
	unsigned long i, x;
	ccTile *tmp;

	context.SetPen(*wxWHITE_PEN);
	context.SetBrush(*wxTRANSPARENT_BRUSH);

	if(map != NULL)	{
		for(i = 0; i < anim->getTileCount(); i++)	{
			tmp = map->getTile(anim->getTile(i));
			tmp->draw(&context, i, 0);
		}
		x = tile * map->getTileWidth();
		context.DrawRectangle(x, 0, map->getTileWidth(), map->getTileHeight());
	}
}

void ccAnimCanvas::update()	{
	SetVirtualSize(map->getTileWidth() * anim->getTileCount(), 
	               map->getTileHeight());
	Refresh();
	tile = 0;
}

void ccAnimCanvas::onLeftDown(wxMouseEvent &event)	{
	int x, y, c;

	event.GetPosition(&x, &y);
	CalcUnscrolledPosition(x, y, &x, &y);

	c = x / map->getTileWidth();

	if(c >= 0 && c < anim->getTileCount())	{
		tile = c;
		Refresh();
	}
}

unsigned long ccAnimCanvas::getTile()	{
	return tile;
}
