/*	animdialog.cc
	Animation dialog functions
	Copyright (C) 2005 Lawrence Sebald

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "animdialog.h"
#include "tilecanvas.h"
#include "ccmap.h"
#include <string>
#include <cstdlib>

extern std::string __uitoa(unsigned int n);

ccAnimDialog::ccAnimDialog(wxWindow *parent, ccMap *m, ccAnim *a) : 
wxDialog(parent, -1, "Animation Editor", wxDefaultPosition, 
wxSize(250, 250), wxDEFAULT_DIALOG_STYLE)	{
	wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL);

	anim = a;
	map = m;

	ok_button = new wxButton(this, wxID_OK, "Ok");
	add_button = new wxButton(this, 777, "Add Tile");
	rem_button = new wxButton(this, 778, "Remove Tile");

	seq_t = new wxStaticText(this, -1, "Tile Sequence:");
	delay_t = new wxStaticText(this, -1, "Animation Delay:");
	delay = new wxTextCtrl(this, -1, __uitoa(a->getDelay()).c_str());

	canvas = new ccAnimCanvas(this, m, a);

	sizer->Add(delay_t, 0, wxALIGN_LEFT);
	sizer->Add(delay, 0, wxALIGN_LEFT);
	sizer->Add(seq_t, 0, wxALIGN_LEFT);
	sizer->Add(canvas, 0, wxALIGN_CENTRE);

	wxBoxSizer *subsizer1 = new wxBoxSizer(wxHORIZONTAL);
	subsizer1->Add(add_button, 0, wxALIGN_CENTRE);
	subsizer1->Add(rem_button, 0, wxALIGN_CENTRE);

	wxBoxSizer *subsizer2 = new wxBoxSizer(wxHORIZONTAL);
	subsizer2->Add(ok_button, 0, wxALIGN_CENTRE);

	sizer->Add(subsizer1, 0, wxALIGN_CENTRE);
	sizer->Add(subsizer2, 0, wxALIGN_CENTRE);

	SetSizer(sizer);
	SetAutoLayout(TRUE);
	Layout();
}

ccAnimDialog::~ccAnimDialog()	{
	delete ok_button;
	delete delay_t;
	delete seq_t;
	delete rem_button;
	delete add_button;
	delete delay;
	delete canvas;
}

BEGIN_EVENT_TABLE(ccAnimDialog, wxDialog)
	EVT_BUTTON(777, ccAnimDialog::onAdd)
	EVT_BUTTON(778, ccAnimDialog::onRemove)
END_EVENT_TABLE()

void ccAnimDialog::onAdd(wxCommandEvent &event)	{
	wxDialog *d = new wxDialog(this, -1, "Select A Tile", wxDefaultPosition, 
	                           wxSize(250, 250), wxDEFAULT_DIALOG_STYLE);

	ccTileCanvas *tc = new ccTileCanvas(d, map, anim);
	wxButton *ok = new wxButton(d, wxID_OK, "Ok");
	wxBoxSizer *bs = new wxBoxSizer(wxVERTICAL);

	bs->Add(tc, 0, wxALIGN_CENTRE);
	bs->Add(ok, 0, wxALIGN_CENTRE);

	d->SetSizer(bs);
	d->SetAutoLayout(true);
	d->Layout();

	d->ShowModal();

	canvas->update();

	delete tc;
	delete ok;
	delete d;
}

void ccAnimDialog::onRemove(wxCommandEvent &event)	{
	if(canvas->getTile() >= anim->getTileCount())
		return;

	anim->removeTile(static_cast<unsigned char>(canvas->getTile()));
	canvas->update();
}

void ccAnimDialog::updateAnimation()	{
	unsigned long d;

	d = strtoul(delay->GetLineText(0).c_str(), NULL, 0);

	anim->setDelay((unsigned short) d);
}
