/*	ccmap.h
	Crabclaw map class definition
	Copyright (C) 2005 Lawrence Sebald

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __CCMAP_H
#define __CCMAP_H

#include <string>
#include <vector>
#include <wx/image.h>
#include <wx/wx.h>

class ccMap;

class ccAnim	{
	private:
		std::vector<unsigned long> tiles;
		unsigned short delay;

	public:
		ccAnim();
		~ccAnim();

		void setDelay(unsigned short d);
		void addTile(unsigned long t);
		void removeTile(unsigned char num);

		unsigned short getDelay();
		unsigned long getTile(int n);
		unsigned char getTileCount();
};

class ccTile	{
	private:
		unsigned long user1;
		unsigned short user2, user3;
		unsigned char user4, user5, user6, user7;
		unsigned char *collision_map;
		wxImage *img;
		wxBitmap bmp;
		unsigned char *txr;
		ccMap *owner;

	public:
		ccTile(ccMap *ownr);
		~ccTile();

		void set32bitUserData(unsigned long val);
		void set16bitUserData(unsigned char num, unsigned short val);
		void set8bitUserData(unsigned char num, unsigned char val);
		void setPassable(unsigned short x, unsigned short y, bool pass);
		void setTexture(wxImage *img, unsigned long txrfmt);

		bool isPassable(unsigned short x, unsigned short y);
		unsigned long get32bitUserData();
		unsigned short get16bitUserData(unsigned char num);
		unsigned char get8bitUserData(unsigned char num);
		unsigned char *getTexture();
		unsigned char *getCollisionMap();

		void setColorKey(unsigned long ck);

		void draw(wxDC *context, unsigned long x, unsigned long y);
		wxBitmap getBitmap();
};

class ccLayer	{
	private:
		unsigned long **map;
		unsigned char scroll;
		ccMap *owner;

	public:
		ccLayer(ccMap *ownr);
		~ccLayer();

		void setSize(unsigned long x, unsigned long y);
		void setScrollSpeed(unsigned char sc);
		void setTileNumber(unsigned long x, unsigned long y, unsigned long t);

		unsigned char getScrollSpeed();
		unsigned long getTileNumber(unsigned long x, unsigned long y);

		void draw(wxDC *context);
		void randomizeLayer();
};

class ccMap	{
	private:
		unsigned long width_px;
		unsigned long height_px;
		unsigned long width_t;
		unsigned long height_t;
		unsigned short tile_width;
		unsigned short tile_height;
		unsigned long version;
		unsigned long txrfmt;
		std::vector<ccTile*> tiles;
		std::vector<ccLayer*> layers;
		std::vector<ccAnim*> anims;
		unsigned long layer_count;
		unsigned long animation_count;
		unsigned long current_layer;
		unsigned long back_layer;
		unsigned long colorkey;

	public:
		ccMap();
		~ccMap();
		void draw(wxDC *context);
		unsigned long getWidth();
		unsigned long getWidthTiles();
		unsigned long getHeight();
		unsigned long getHeightTiles();
		unsigned long getTileWidth();
		unsigned long getTileHeight();
		void setWidth(unsigned long w);
		void setHeight(unsigned long h);
		void setTileWidth(unsigned short w);
		void setTileHeight(unsigned short h);
		std::string getVersionString();	
		void setVersion(unsigned short maj, unsigned char min, unsigned char mic);
		ccTile *getTile(unsigned long num);
		unsigned long getTileCount();
		void setTileAt(unsigned long x, unsigned long y, unsigned long t);
		void setTextureFormat(unsigned long fmt);
		void setColorKey(unsigned long ck);
		unsigned long getColorKey();
		void addLayer(ccLayer *lay, bool inc = true);
		unsigned long getLayerCount();
		void setCurrentLayer(unsigned long layer);
		void setBackLayer(unsigned long layer);
		void addAnim(ccAnim *an, bool inc = true);
		unsigned long getAnimCount();
		ccAnim *getAnim(unsigned long num);
		void exportToPNG(const char *fn);

		void loadTiles(const char *fn);

		void save(std::string fn);

		static ccMap *load(std::string fn);
};

#endif
