/*	collidedialog.cc
	Collision map editing dialog
	Copyright (C) 2005 Lawrence Sebald

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "collidedialog.h"
#include "ccmap.h"

class __ccCollisionCanvas : public wxScrolledWindow	{
	private:
		ccTile *tile;
		ccCollisionDialog *par;
		int x;
		int y;

	protected:
		DECLARE_EVENT_TABLE()

	public:
		__ccCollisionCanvas(ccCollisionDialog *parent, ccTile *t);
		~__ccCollisionCanvas();
		void onLeftDown(wxMouseEvent &event);
		void onRightDown(wxMouseEvent &event);
		void OnDraw(wxDC &context);
		int getX() { return x; }
		int getY() { return y; }
		void setX(int xp) { x = xp; }
		void setY(int yp) { y = yp; }
};

__ccCollisionCanvas::__ccCollisionCanvas(ccCollisionDialog *parent, ccTile *t) :
wxScrolledWindow(parent, -1, wxDefaultPosition, wxSize(128, 128))	{
	par = parent;
	x = y = 0;
	tile = t;

	SetVirtualSize(128, 128);
	SetScrollRate(1, 1);
	Refresh();
	par->updateBox(0, 0);
}

__ccCollisionCanvas::~__ccCollisionCanvas()	{
}

BEGIN_EVENT_TABLE(__ccCollisionCanvas, wxScrolledWindow)
	EVT_LEFT_DOWN(__ccCollisionCanvas::onLeftDown)
	EVT_RIGHT_DOWN(__ccCollisionCanvas::onRightDown)
END_EVENT_TABLE()

void __ccCollisionCanvas::onLeftDown(wxMouseEvent &event)	{
	event.GetPosition(&x, &y);
	CalcUnscrolledPosition(x, y, &x, &y);

	x /= 128 / tile->getBitmap().GetWidth();
	y /= 128 / tile->getBitmap().GetHeight();

	Refresh();
	par->updateBox(x, y);
}

void __ccCollisionCanvas::onRightDown(wxMouseEvent &event)	{
	event.GetPosition(&x, &y);
	CalcUnscrolledPosition(x, y, &x, &y);

	x /= 128 / tile->getBitmap().GetWidth();
	y /= 128 / tile->getBitmap().GetHeight();

	Refresh();
	tile->setPassable(x, y, tile->isPassable(x, y) ^ 1);
	par->updateBox(x, y);
}

void __ccCollisionCanvas::OnDraw(wxDC &context)	{
	wxBitmap bmp = tile->getBitmap();
	int ratio1 = 128 / bmp.GetWidth();
	int ratio2 = 128 / bmp.GetHeight();
	wxImage img = bmp.ConvertToImage();

	context.SetPen(*wxWHITE_PEN);
	context.SetBrush(*wxTRANSPARENT_BRUSH);

	bmp = wxBitmap(img.Scale(128, 128));
	context.DrawBitmap(bmp, 0, 0, true);
	context.DrawRectangle(x * ratio1 - 1, y * ratio2 - 1, ratio1 + 2,
	                      ratio2 + 2);
}

ccCollisionDialog::ccCollisionDialog(wxWindow *parent, ccTile *t) : 
wxDialog(parent, -1, "Collision Map Editor", wxDefaultPosition, 
wxSize(250, 250), wxDEFAULT_DIALOG_STYLE)	{
	wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL);
	wxString choices[2] = { "No", "Yes" };

	tile = t;

	passable = new wxChoice(this, 777, wxDefaultPosition, wxDefaultSize, 2,
	                        choices);
	canvas = new __ccCollisionCanvas(this, t);
	ok_button = new wxButton(this, wxID_OK, "Ok");
	t1 = new wxStaticText(this, -1, "Tile Bitmap:");
	t2 = new wxStaticText(this, -1, "Passable:");

	sizer->Add(t1, 0, wxALIGN_CENTRE);
	sizer->Add(canvas, 0, wxALIGN_CENTRE);
	sizer->Add(t2, 0, wxALIGN_CENTRE);
	sizer->Add(passable, 0, wxALIGN_CENTRE);
	sizer->Add(ok_button, 0, wxALIGN_CENTRE);

	SetSizer(sizer);
	Layout();
}

ccCollisionDialog::~ccCollisionDialog()	{
	delete canvas;
	delete ok_button;
	delete t1;
	delete t2;
}

BEGIN_EVENT_TABLE(ccCollisionDialog, wxDialog)
	EVT_CHOICE(777, ccCollisionDialog::onChoice)
END_EVENT_TABLE()


void ccCollisionDialog::updateBox(int x, int y)	{
	if(tile->isPassable(x, y))
		passable->SetSelection(1);
	else
		passable->SetSelection(0);
}

void ccCollisionDialog::onChoice(wxCommandEvent &event)	{
	tile->setPassable(canvas->getX(), canvas->getY(), passable->GetSelection());
}
