/*	editframe.cc
	Main window and its functions
	Copyright (C) 2005 Lawrence Sebald

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "editframe.h"
#include "aboutdialog.h"
#include "animdialog.h"
#include "layerdialog.h"
#include "tiledialog.h"
#include "collidedialog.h"
#include "newmap.h"
#include <iostream>
#include <wx/colordlg.h>

#include "wizard.xpm"

using namespace std;

ccEditFrame::ccEditFrame(int x, int y, int w, int h) :
wxFrame(NULL, -1, "Crabclaw Map Editor", wxPoint(x, y), wxSize(w, h))	{
	CreateStatusBar();
	SetStatusText("Welcome to Crabclaw Map Editor 0.9.2");

	menubar = new wxMenuBar();
	filemenu = new wxMenu();
	mapmenu = new wxMenu();
	layermenu = new wxMenu();
	tilemenu = new wxMenu();
	animmenu = new wxMenu();
	helpmenu = new wxMenu();

	filemenu->Append(FILE_NEW, "&New", "New map");
	filemenu->Append(FILE_OPEN, "&Open", "Open Map");
	filemenu->Append(FILE_SAVE, "&Save", "Save Map");
	filemenu->Append(FILE_SAVEAS, "Save &As", "Save Map");
	filemenu->AppendSeparator();
	filemenu->Append(FILE_IMPORT, "Import &Tileset", "Import tiles");
	filemenu->Append(FILE_EXPORT, "&Export Map", "Save map to an image file");
#ifndef __WXMAC__
	filemenu->AppendSeparator();
	filemenu->Append(FILE_QUIT, "&Quit", "Quit the application");
#endif
	mapmenu->Append(MAP_COLORKEY, "Set Color &Key", "Set this map's color key");
	mapmenu->AppendCheckItem(MAP_SHOWANIMS, "Show &Animations",
	                         "Switches between animation and tile display");
	layermenu->Append(LAYER_SET, "&Set Current Layer", 
	                  "Sets the displayed layer");
	layermenu->Append(LAYER_SETB, "Set &Back Layer", 
	                  "Sets the back layer (displayed under the current layer)");
	layermenu->AppendSeparator();
	layermenu->Append(LAYER_ADD, "Add New &Layer", "Adds a new layer");
	tilemenu->Append(TILE_PROPS, "Tile &Properties", 
	                 "Adjust settings of the current tile");
	tilemenu->Append(TILE_COLLISION, "Collision &Map", 
	                 "Set the collision map for the current tile.");
	animmenu->Append(ANIM_NEW, "&New Animation", "Adds a new animation");
	animmenu->Append(ANIM_EDIT, "&Edit Animation", "Edit the current animation");
	helpmenu->Append(HELP_ABOUT, "&About", "About this application");
	menubar->Append(filemenu, "&File");
	menubar->Append(mapmenu, "&Map");
	menubar->Append(tilemenu, "&Tile");
	menubar->Append(animmenu, "&Animations");
	menubar->Append(layermenu, "&Layer");
	menubar->Append(helpmenu, "&Help");
	SetMenuBar(menubar);
	wxBoxSizer *sizer = new wxBoxSizer(wxHORIZONTAL);
	canvas = new ccMapCanvas(this);
	tile_canv = new ccTileCanvas(this, canvas);
	sizer->Add(canvas, 1, wxGROW | wxSHRINK | wxADJUST_MINSIZE);
	sizer->Add(tile_canv, 0, wxGROW | wxSHRINK | wxADJUST_MINSIZE);
	SetSizer(sizer);
	wxInitAllImageHandlers();
	map = NULL;
	curfn = "";
}

ccEditFrame::~ccEditFrame()	{
}

BEGIN_EVENT_TABLE(ccEditFrame, wxFrame)
	EVT_MENU(FILE_NEW, ccEditFrame::onFileNew)
	EVT_MENU(FILE_OPEN, ccEditFrame::onFileOpen)
	EVT_MENU(FILE_SAVE, ccEditFrame::onFileSave)
	EVT_MENU(FILE_SAVEAS, ccEditFrame::onFileSaveAs)
	EVT_MENU(FILE_IMPORT, ccEditFrame::onFileImport)
	EVT_MENU(FILE_EXPORT, ccEditFrame::onFileExport)
	EVT_MENU(FILE_QUIT, ccEditFrame::onFileQuit)
	EVT_MENU(MAP_COLORKEY, ccEditFrame::onMapColorkey)
	EVT_MENU(MAP_SHOWANIMS, ccEditFrame::onMapShowAnims)
	EVT_MENU(LAYER_SET, ccEditFrame::onLayerSet)
	EVT_MENU(LAYER_SETB, ccEditFrame::onLayerSetBack)
	EVT_MENU(LAYER_ADD, ccEditFrame::onLayerAdd)
	EVT_MENU(TILE_PROPS, ccEditFrame::onTileProperties)
	EVT_MENU(TILE_COLLISION, ccEditFrame::onTileCollision)
	EVT_MENU(ANIM_NEW, ccEditFrame::onAnimNew)
	EVT_MENU(ANIM_EDIT, ccEditFrame::onAnimEdit)
	EVT_MENU(HELP_ABOUT, ccEditFrame::onHelpAbout)

	EVT_WIZARD_CANCEL(-1, ccEditFrame::onWizCancel)
	EVT_WIZARD_FINISHED(-1, ccEditFrame::onWizFinished)
END_EVENT_TABLE()

void ccEditFrame::onFileNew(wxCommandEvent &event)	{
	wxWizard *wiz = new wxWizard(this, -1, "New Map Wizard", wxBitmap(wizard_xpm));
	ccMapContainer *cont = new ccMapContainer();

	wxWizardPageSimple *p1 = new wxWizardPageSimple(wiz);
	wxStaticText *text = new wxStaticText(p1, -1, "Welcome to the New Map wizard.\n\n"
                                                  "This wizard will help you to create a\n"
	                                              "new Crabclaw Map.", wxPoint(5, 5));
	ccWizPage2 *p2 = new ccWizPage2(wiz, cont);
	ccWizPage4 *p4 = new ccWizPage4(wiz, cont);
	ccWizPage3 *p3 = new ccWizPage3(wiz, cont, p4);

	wxWizardPageSimple::Chain(p1, p2);
	wxWizardPageSimple::Chain(p2, p3);
	wxWizardPageSimple::Chain(p3, p4);

	wxSize sz = text->GetBestSize();
	wxSize sizes[3];

	sizes[0] = p2->GetSizer()->GetSize();
	sizes[1] = p3->GetSizer()->GetSize();
	sizes[2] = p4->GetSizer()->GetSize();

	for(int i = 0; i < 3; i++)	{
		if(sz.GetWidth() < sizes[i].GetWidth())
			sz.SetWidth(sizes[i].GetWidth());
		if(sz.GetHeight() < sizes[i].GetHeight())
			sz.SetHeight(sizes[i].GetHeight());
	}

	p2->GetSizer()->SetMinSize(sz);
	p2->GetSizer()->Fit(p2);
	p3->GetSizer()->SetMinSize(sz);
	p3->GetSizer()->Fit(p3);
	p4->GetSizer()->SetMinSize(sz);
	p4->GetSizer()->Fit(p4);

	wiz->SetPageSize(sz);
	if(!wiz->RunWizard(p1))	{
		wiz->Destroy();
		return;
	}

	if(map != NULL)	{
		delete map;
	}

	map = new ccMap();
	map->setTileWidth(cont->getTileWidth());
	map->setTileHeight(cont->getTileHeight());
	map->setWidth(cont->getWidth());
	map->setHeight(cont->getHeight());
	map->setVersion(cont->getVersion() >> 16, (cont->getVersion() >> 8) & 0xFF, 
	                cont->getVersion() & 0xFF);
	map->setTextureFormat(cont->getTileFormat());
	map->loadTiles(cont->getFilename().c_str());

	canvas->setMap(map);
	tile_canv->setMap(map);

	wiz->Destroy();
}

void ccEditFrame::onFileOpen(wxCommandEvent &event)	{
	wxFileDialog *fd = new wxFileDialog(this, "Select a Map", "", "",
	                                    "Crabclaw maps|*.ccm", 
	                                    wxOPEN | wxHIDE_READONLY);
	int res = fd->ShowModal();

	if(res == wxID_OK)	{
		map = ccMap::load(fd->GetPath().c_str());
		canvas->setMap(map);
		tile_canv->setMap(map);
		curfn = fd->GetPath().c_str();
	}

	delete fd;
}

void ccEditFrame::onFileSave(wxCommandEvent &event)	{
	if(curfn == "")	{
		onFileSaveAs(event);
		return;
	}
	map->save(curfn);
}

void ccEditFrame::onFileSaveAs(wxCommandEvent &event)	{
	wxFileDialog *fd = new wxFileDialog(this, "Save Map", "", "",
	                                    "Crabclaw maps|*.ccm", 
	                                    wxSAVE | wxHIDE_READONLY);

	if(map == NULL)	{
		delete fd;
		return;
	}

	int res = fd->ShowModal();

	if(res == wxID_OK)	{
		curfn = fd->GetPath();
		onFileSave(event);
	}

	delete fd;
}

void ccEditFrame::onFileImport(wxCommandEvent &event)	{
	wxFileDialog *fd = new wxFileDialog(this, "Select a tileset", "", "",
	                                    "PNG Images|*.png", 
	                                    wxOPEN | wxHIDE_READONLY);

	if(map == NULL)	{
		delete fd;
		return;
	}

	int res = fd->ShowModal();

	if(res == wxID_OK)	{
		map->loadTiles(fd->GetPath().c_str());
	}

	delete fd;
}

void ccEditFrame::onFileExport(wxCommandEvent &event)	{
	wxFileDialog *fd = new wxFileDialog(this, "Export Map", "", "",
	                                    "PNG Images|*.png", 
	                                    wxSAVE | wxHIDE_READONLY);

	if(!map)	{
		delete fd;
		return;
	}

	int res = fd->ShowModal();

	if(res == wxID_OK)	{
		map->exportToPNG(fd->GetPath().c_str());
	}

	delete fd;
}

void ccEditFrame::onFileQuit(wxCommandEvent &event)	{
	Close(FALSE);
}

void ccEditFrame::onMapColorkey(wxCommandEvent &event)	{
	if(map == NULL)	{
		return;
	}

	wxColourDialog *cd;
	wxColourData cold;
	wxColour c;
	unsigned long col;

	col = map->getColorKey();
	c.Set((col >> 16) & 0xFF, (col >> 8) & 0xFF, (col) & 0xFF);
	cold.SetColour(c);
	cd = new wxColourDialog(this, &cold);

	if(cd->ShowModal() == wxID_OK)	{
		c = cd->GetColourData().GetColour();
		col = (c.Red() << 16) | (c.Green() << 8) | (c.Blue());
		map->setColorKey(col);
	}

	Refresh();

	delete cd;
}

void ccEditFrame::onMapShowAnims(wxCommandEvent &event)	{
	if(!map)
		return;

	if(event.IsChecked())
		tile_canv->animationMode(true);
	else
		tile_canv->animationMode(false);
}

void ccEditFrame::onHelpAbout(wxCommandEvent &event)	{
	ccAboutDialog *about = new ccAboutDialog(this);
	about->ShowModal();
	delete about;
}

void ccEditFrame::onWizFinished(wxWizardEvent& event)	{
	wxMessageBox("Your map was created successfully.", "Map Created", 
	             wxICON_INFORMATION | wxOK);
}

void ccEditFrame::onWizCancel(wxWizardEvent &event)	{
	int ret = wxMessageBox("Do you really want to cancel the wizard?", 
	                       "Really Cancel?", wxYES_NO | wxICON_QUESTION);
	if(ret == wxYES)
		wxMessageBox("New map creation terminated", "Information", wxICON_ERROR);
	else
		event.Veto();
}

void ccEditFrame::onLayerSet(wxCommandEvent &event)	{
	if(map == NULL)
		return;

	ccLayerDialog *dlg = new ccLayerDialog(this, map->getLayerCount());

	if(dlg->ShowModal() == wxID_OK)	{
		if(dlg->getSelectedLayer() > -1)
			map->setCurrentLayer(dlg->getSelectedLayer());
	}

	canvas->Refresh();

	delete dlg;
}

void ccEditFrame::onLayerSetBack(wxCommandEvent &event)	{
	if(map == NULL)
		return;

	ccLayerDialog *dlg = new ccLayerDialog(this, map->getLayerCount(), true);

	if(dlg->ShowModal() == wxID_OK)	{
		if(dlg->getSelectedLayer() > -1)
			map->setBackLayer(dlg->getSelectedLayer());
	}

	canvas->Refresh();

	delete dlg;
}

void ccEditFrame::onLayerAdd(wxCommandEvent &event)	{
	if(map == NULL)
		return;

	ccLayer *layer = new ccLayer(map);

	map->addLayer(layer);
}

void ccEditFrame::onTileProperties(wxCommandEvent &event)	{
	if(map == NULL)
		return;

	ccTile *t = map->getTile(canvas->getCurrentTile());
	ccTileDialog *tiled = new ccTileDialog(this, t);

	if(tiled->ShowModal() == wxID_OK)
		tiled->updateTile();

	delete tiled;
}

void ccEditFrame::onTileCollision(wxCommandEvent &event)	{
	if(!map)
		return;

	ccTile *t = map->getTile(canvas->getCurrentTile());
	ccCollisionDialog *dlg = new ccCollisionDialog(this, t);

	dlg->ShowModal();

	delete dlg;
}

void ccEditFrame::onAnimNew(wxCommandEvent &event)	{
	if(!map)
		return;

	ccAnim *newanim = new ccAnim();

	ccAnimDialog *d = new ccAnimDialog(this, map, newanim);
	d->ShowModal();

	d->updateAnimation();

	map->addAnim(newanim);
	delete d;
}

void ccEditFrame::onAnimEdit(wxCommandEvent &event)	{
	ccAnim *anim;

	if(!map)
		return;

	if(tile_canv->isInAnimationMode())	{
		anim = map->getAnim(canvas->getCurrentTile() & 0x7FFFFFFF);
		ccAnimDialog *d = new ccAnimDialog(this, map, anim);
		d->ShowModal();
		d->updateAnimation();
		delete d;
	}
}
