/*	mapcanvas.cc
	Map canvas functions
	Copyright (C) 2005 Lawrence Sebald

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "mapcanvas.h"
#include "editframe.h"

ccMapCanvas::ccMapCanvas(ccEditFrame *parent) :
wxScrolledWindow(parent, -1, wxDefaultPosition, wxSize(480, 480))	{
	map = NULL;
	p = parent;
	tile = 6;
	ignoreMouse = false;
}

ccMapCanvas::~ccMapCanvas()	{
	delete map;
}

BEGIN_EVENT_TABLE(ccMapCanvas, wxScrolledWindow)
	EVT_LEFT_DOWN(ccMapCanvas::onLeftDown)
	EVT_MOTION(ccMapCanvas::onMotion)
	EVT_ENTER_WINDOW(ccMapCanvas::onMouseIn)
	EVT_LEAVE_WINDOW(ccMapCanvas::onMouseOut)
END_EVENT_TABLE()

void ccMapCanvas::setMap(ccMap *m)	{
	map = m;
	SetVirtualSize(m->getWidth(), m->getHeight());
	SetScrollRate(m->getTileWidth(), m->getTileHeight());
	Refresh();
}

void ccMapCanvas::OnDraw(wxDC &context)	{
	if(map != NULL)	{
		map->draw(&context);
	}
}

void ccMapCanvas::onLeftDown(wxMouseEvent &event)	{
	int x, y;

	if(ignoreMouse || map == NULL)	{
		return;
	}

	event.GetPosition(&x, &y);
	CalcUnscrolledPosition(x, y, &x, &y);

	if(x > 0 && y > 0 && (unsigned long)x < map->getWidth() && 
	   (unsigned long)y < map->getHeight())	{
		map->setTileAt(x / map->getTileWidth(), y / map->getTileHeight(), tile);
		Refresh();
	}
}

void ccMapCanvas::onMotion(wxMouseEvent &event)	{
	int x, y;

	if(!event.LeftIsDown() || ignoreMouse || map == NULL)	{
		return;
	}

	event.GetPosition(&x, &y);
	CalcUnscrolledPosition(x, y, &x, &y);

	if(x > 0 && y > 0 && (unsigned long)x < map->getWidth() && 
	   (unsigned long)y < map->getHeight())	{
		map->setTileAt(x / map->getTileWidth(), y / map->getTileHeight(), tile);
		Refresh();
	}
}

void ccMapCanvas::onMouseIn(wxMouseEvent &event)	{
	ignoreMouse = false;
}

void ccMapCanvas::onMouseOut(wxMouseEvent &event)	{
	ignoreMouse = true;
}

void ccMapCanvas::setCurrentTile(unsigned long t)	{
	tile = t;
}

unsigned long ccMapCanvas::getCurrentTile()	{
	return tile;
}
