/*	newmap.cc
	New map wizard
	Copyright (C) 2005 Lawrence Sebald

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "newmap.h"

extern std::string __uitoa(unsigned int n);

static wxString getVersionString(unsigned long version)	{
	wxString str = "";
	str << __uitoa(version >> 16).c_str() << '.';
	str << __uitoa((version >> 8) & 0xFF).c_str() << '.';
	str << __uitoa(version & 0xFF).c_str();

	return str;
}

ccMapContainer::ccMapContainer()	{
	width = 0;
	height = 0;
	version = 0;
	tile_w = 0;
	tile_h = 0;
	tile_fmt = 0;
	filename = "";
}

ccMapContainer::~ccMapContainer()	{
}

void ccMapContainer::setPage2(unsigned long w, unsigned long h, unsigned long v)	{
	width = w;
	height = h;
	version = v;
}

void ccMapContainer::setPage3(unsigned short w, unsigned short h, unsigned long fl, std::string fn)	{
	tile_w = w;
	tile_h = h;
	tile_fmt = fl;
	filename = fn;
}

ccWizPage2::ccWizPage2(wxWizard *parent, ccMapContainer *cont) : wxWizardPageSimple(parent)	{
	wxString choices[] = { "0.9.2" };
	width = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxDefaultSize, 
	                       wxTE_PROCESS_TAB);
	height = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxDefaultSize, 
	                        wxTE_PROCESS_TAB);
	version = new wxComboBox(this, -1, "Version", wxDefaultPosition, 
	                         wxDefaultSize, 1, choices, wxCB_DROPDOWN |
							 wxCB_READONLY);
	wxStaticText* t = new wxStaticText(this, -1, "First, I need some basic information\n"
	                                             "about the map you wish to create.");
	wxBoxSizer *s1 = new wxBoxSizer(wxVERTICAL);
	wxGridSizer *s2 = new wxGridSizer(3, 2, 10, 10);

	s2->Add(new wxStaticText(this, -1, "Width of Map (in tiles):"));
	s2->Add(width);
	s2->Add(new wxStaticText(this, -1, "Height of Map (in tiles):"));
	s2->Add(height);
	s2->Add(new wxStaticText(this, -1, "Crabclaw Map Version:"));
	s2->Add(version);

	s1->Add(t);
	s1->Add(s2);

	SetSizer(s1);

	s1->Fit(this);
	container = cont;
}

ccWizPage2::~ccWizPage2()	{
	delete width;
	delete height;
	delete version;
}

bool ccWizPage2::TransferDataFromWindow()	{
	wxString w, h;
	unsigned long wi, he;

	w = width->GetLineText(0);
	h = height->GetLineText(0);

	if(!w.compare("") || !h.compare(""))	{
		wxMessageBox("You must fill in all fields.", "Information", wxICON_ERROR);
		return false;
	}
	else if(version->GetSelection() == -1)	{
		wxMessageBox("You must select a version.", "Information", wxICON_ERROR);
		return false;
	}

	wi = strtoul(w.c_str(), NULL, 0);
	he = strtoul(h.c_str(), NULL, 0);

	if(wi == 0 || he == 0)	{
		wxMessageBox("You must input a positive number for width and height.", "Informtaion", wxICON_ERROR);
		return false;
	}

	container->setPage2(wi, he, 0x00000902);
	return true;
}

ccWizPage3::ccWizPage3(wxWizard *parent, ccMapContainer *cont, ccWizPage4 *wp4) : wxWizardPageSimple(parent)	{
	wxString choices[] = { "RGB565", "RGB888", "RGB555"	};
	width = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxDefaultSize, 
	                       wxTE_PROCESS_TAB);
	height = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxDefaultSize, 
	                        wxTE_PROCESS_TAB);
	filename = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxDefaultSize, 
	                          wxTE_PROCESS_TAB);
	format = new wxComboBox(this, -1, "Format", wxDefaultPosition, 
	                        wxDefaultSize, 3, choices, wxCB_DROPDOWN |
							wxCB_READONLY);

	wxStaticText* t = new wxStaticText(this, -1, "Now, I need some information about\n"
	                                             "the tiles you will use in your map.");
	wxBoxSizer *s1 = new wxBoxSizer(wxVERTICAL);
	wxBoxSizer *s3 = new wxBoxSizer(wxHORIZONTAL);
	wxGridSizer *s2 = new wxGridSizer(4, 2, 10, 10);

	s3->Add(filename);
	s3->Add(new wxButton(this, 777, "Browse"));

	s2->Add(new wxStaticText(this, -1, "Width of a Tile (in pixels):"));
	s2->Add(width);
	s2->Add(new wxStaticText(this, -1, "Height of a Tile (in pixels):"));
	s2->Add(height);
	s2->Add(new wxStaticText(this, -1, "Tileset Filename:"));
	s2->Add(s3);
	s2->Add(new wxStaticText(this, -1, "Texture Format:"));
	s2->Add(format);

	s1->Add(t);
	s1->Add(s2);

	SetSizer(s1);

	s1->Fit(this);

	container = cont;
	wizp4 = wp4;
}

BEGIN_EVENT_TABLE(ccWizPage3, wxWizardPageSimple)
	EVT_BUTTON(777, ccWizPage3::onBrowse)
END_EVENT_TABLE()

ccWizPage3::~ccWizPage3()	{
	delete width;
	delete height;
	delete filename;
	delete format;
}

bool ccWizPage3::TransferDataFromWindow()	{
	wxString w, h, fn;
	unsigned long wi, he;
	unsigned long fmt = 0;
	FILE *fp;

	w = width->GetLineText(0);
	h = height->GetLineText(0);
	fn = filename->GetLineText(0);

	if(!w.compare("") || !h.compare("") || !fn.compare(""))	{
		wxMessageBox("You must fill in all fields.", "Information", wxICON_ERROR);
		return false;
	}
	else if(format->GetSelection() == -1)	{
		wxMessageBox("You must select a tile format.", "Information", wxICON_ERROR);
		return false;
	}
	else if((fp = fopen(fn.c_str(), "rb")) == NULL)	{
		wxMessageBox("Cannot open tileset.", "Information", wxICON_ERROR);
		return false;
	}

	fclose(fp);

	wi = strtoul(w.c_str(), NULL, 0);
	he = strtoul(h.c_str(), NULL, 0);

	if(wi == 0 || he == 0)	{
		wxMessageBox("You must input a positive number for width and height.", "Informtaion", wxICON_ERROR);
		return false;
	}

	switch(format->GetSelection())	{
		case 0:
			fmt = 1;
			break;
		case 1:
			fmt = 2;
			break;
		case 2:
			fmt = 3;
			break;
		default:
			printf("We shouldn't get here....\n");
			return false;
	}

	container->setPage3(wi, he, fmt, std::string(fn.c_str()));
	wizp4->update();

	return true;
}

void ccWizPage3::onBrowse(wxCommandEvent &event)	{
	wxFileDialog *fd = new wxFileDialog(this, "Select a Tileset", "", "",
	                                    "PNG Files|*.png", wxOPEN | wxHIDE_READONLY);
	int res = fd->ShowModal();

	if(res == wxID_OK)	{
		filename->Clear();
		filename->WriteText(fd->GetPath());
	}
}

ccWizPage4::ccWizPage4(wxWizard *parent, ccMapContainer *cont) : wxWizardPageSimple(parent)	{
	wxBoxSizer *s = new wxBoxSizer(wxVERTICAL);
	wxGridSizer *s2 = new wxGridSizer(2, 10, 10);

	w = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxDefaultSize, wxTE_READONLY);
	h = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxDefaultSize, wxTE_READONLY);
	tw = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxDefaultSize, wxTE_READONLY);
	th = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxDefaultSize, wxTE_READONLY);
	v = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxDefaultSize, wxTE_READONLY);
	tf = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxDefaultSize, wxTE_READONLY);
	fn = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxDefaultSize, wxTE_READONLY);

	s2->Add(new wxStaticText(this, -1, "Map Width:"));
	s2->Add(w);
	s2->Add(new wxStaticText(this, -1, "Map Height:"));
	s2->Add(h);
	s2->Add(new wxStaticText(this, -1, "Map Version:"));
	s2->Add(v);
	s2->Add(new wxStaticText(this, -1, "Tile Width:"));
	s2->Add(tw);
	s2->Add(new wxStaticText(this, -1, "Tile Height:"));
	s2->Add(th);
	s2->Add(new wxStaticText(this, -1, "Tileset:"));
	s2->Add(fn);
	s2->Add(new wxStaticText(this, -1, "Texture Format:"));
	s2->Add(tf);

	s->Add(new wxStaticText(this, -1, "Please confirm that the information on this\n"
	                                  "page is correct.\n"
	                                  "If not, use the back button to correct the information."));
	s->Add(s2);
	SetSizer(s);
	s->Fit(this);
	container = cont;
}

ccWizPage4::~ccWizPage4()	{
	delete w;
	delete h;
	delete tw;
	delete th;
	delete v;
	delete tf;
	delete fn;
}

void ccWizPage4::update()	{
	wxString tmp = "";

	w->WriteText(__uitoa(container->getWidth()).c_str());
	h->WriteText(__uitoa(container->getHeight()).c_str());
	tw->WriteText(__uitoa(container->getTileWidth()).c_str());
	th->WriteText(__uitoa(container->getTileHeight()).c_str());
	v->WriteText(getVersionString(container->getVersion()));
	switch(container->getTileFormat())	{
		case 1:
			tmp = "RGB565";
			break;
		case 2:
			tmp = "RGB888";
			break;
		case 3:
			tmp = "RGB555";
			break;
		default:
			printf("Shouldn't get here...\n");
	}
	tf->WriteText(tmp);
	fn->WriteText(container->getFilename().c_str());
}
