/*	tilecanvas.cc
	Tile canvas
	Copyright (C) 2005 Lawrence Sebald

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "mapcanvas.h"
#include "tilecanvas.h"
#include "editframe.h"

ccTileCanvas::ccTileCanvas(ccEditFrame *parent, ccMapCanvas *canvas) :
wxScrolledWindow(parent, -1, wxDefaultPosition, wxSize(150, 480))	{
	map = NULL;
	p = parent;
	tile = 0;
	ignoreMouse = false;
	can = canvas;
	rows = cols = 0;
	animMode = false;
}

ccTileCanvas::ccTileCanvas(wxDialog *parent, ccMap *m, ccAnim *a) :
wxScrolledWindow(parent, -1, wxDefaultPosition, wxSize(200, 200))	{
	int x, y;

	map = m;
	anim = a;
	tile = 0;
	ignoreMouse = false;
	can = NULL;
	animMode = false;

	GetSize(&x, &y);
	cols = (x - 16) / m->getTileWidth();
	if(cols <= 0)
		cols = 1;
	rows = m->getTileCount() / cols;
	SetVirtualSize(m->getTileWidth() * cols, m->getTileHeight() * rows);
	SetScrollRate(m->getTileWidth(), m->getTileHeight());
	Refresh();
}

ccTileCanvas::~ccTileCanvas()	{
}

BEGIN_EVENT_TABLE(ccTileCanvas, wxScrolledWindow)
	EVT_LEFT_DOWN(ccTileCanvas::onLeftDown)
	EVT_ENTER_WINDOW(ccTileCanvas::onMouseIn)
	EVT_LEAVE_WINDOW(ccTileCanvas::onMouseOut)
END_EVENT_TABLE()

void ccTileCanvas::setMap(ccMap *m)	{
	int x, y;
	map = m;
	GetSize(&x, &y);
	cols = x / m->getTileWidth();
	if(cols <= 0)
		cols = 1;

	rows = m->getTileCount() / cols;
	if(rows <= 0)
		rows = 1;

	SetVirtualSize(m->getTileWidth() * cols, m->getTileHeight() * rows);
	SetScrollRate(m->getTileWidth(), m->getTileHeight());
	Refresh();
}

void ccTileCanvas::OnDraw(wxDC &context)	{
	unsigned long i, x, y;
	ccTile *tmp;

	context.SetPen(*wxWHITE_PEN);
	context.SetBrush(*wxTRANSPARENT_BRUSH);
	if(map != NULL)	{
		if(!animMode)	{
			for(i = 0; i < map->getTileCount(); i++)	{
				tmp = map->getTile(i);
				tmp->draw(&context, i % cols, i / cols);
			}
		}
		else	{
			for(i = 0; i < map->getAnimCount(); i++)	{
				tmp = map->getTile(i | 0x80000000);
				tmp->draw(&context, i % cols, i / cols);
			}
		}
		x = tile % cols * map->getTileWidth();
		y = tile / cols * map->getTileHeight();
		context.DrawRectangle(x, y, map->getTileWidth(), map->getTileHeight());
	}
}

void ccTileCanvas::onLeftDown(wxMouseEvent &event)	{
	int x, y, r, c;

	if(ignoreMouse || map == NULL)	{
		return;
	}

	event.GetPosition(&x, &y);
	CalcUnscrolledPosition(x, y, &x, &y);

	r = y / map->getTileHeight();
	c = x / map->getTileWidth();

	if(r >= 0 && c >= 0 && r < rows && c < cols)	{
		tile = r * cols + c;
		if(can)
			if(!animMode)
				can->setCurrentTile(tile);
			else
				can->setCurrentTile(tile | 0x80000000);
		else	{
			anim->addTile(tile);
		}
		Refresh();
	}
}

void ccTileCanvas::onMouseIn(wxMouseEvent &event)	{
	ignoreMouse = false;
}

void ccTileCanvas::onMouseOut(wxMouseEvent &event)	{
	ignoreMouse = true;
}

void ccTileCanvas::animationMode(bool on)	{
	int x, y;

	if(!can)
		return;

	animMode = on;

	GetSize(&x, &y);
	cols = x / map->getTileWidth();
	if(cols <= 0)
		cols = 1;

	if(!on)
		rows = map->getTileCount() / cols;
	else
		rows = map->getAnimCount() / cols;

	if(rows <= 0)
		rows = 1;

	SetVirtualSize(map->getTileWidth() * cols, map->getTileHeight() * rows);
	SetScrollRate(map->getTileWidth(), map->getTileHeight());
	tile = 0;

	if(on)
		can->setCurrentTile(0x80000000);
	else
		can->setCurrentTile(0);

	Refresh();
}

bool ccTileCanvas::isInAnimationMode()	{
	return animMode;
}
