/*	tiledialog.cc
	Tile dialog
	Copyright (C) 2005 Lawrence Sebald

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "tiledialog.h"
#include "ccmap.h"
#include <string>
#include <cstdlib>

extern std::string __uitoa(unsigned int n);

ccTileDialog::ccTileDialog(wxWindow *parent, ccTile *t) : 
wxDialog(parent, -1, "Tile Data Editor", wxDefaultPosition, 
         wxSize(250, 400), wxDEFAULT_DIALOG_STYLE)	{
	wxGridSizer *sizer = new wxGridSizer(8, 2, 5, 5);

	tile = t;
	ok_button = new wxButton(this, wxID_OK, "Ok");
	cancel_button = new wxButton(this, wxID_CANCEL, "Cancel");

	u32_1 = new wxTextCtrl(this, -1, __uitoa(t->get32bitUserData()).c_str());
	u16_1 = new wxTextCtrl(this, -1, __uitoa(t->get16bitUserData(1)).c_str());
	u16_2 = new wxTextCtrl(this, -1, __uitoa(t->get16bitUserData(2)).c_str());
	u8_1 = new wxTextCtrl(this, -1, __uitoa(t->get8bitUserData(1)).c_str());
	u8_2 = new wxTextCtrl(this, -1, __uitoa(t->get8bitUserData(2)).c_str());
	u8_3 = new wxTextCtrl(this, -1, __uitoa(t->get8bitUserData(3)).c_str());
	u8_4 = new wxTextCtrl(this, -1, __uitoa(t->get8bitUserData(4)).c_str());
	t_u32_1 = new wxStaticText(this, -1, "32-bit Data:");
	t_u16_1 = new wxStaticText(this, -1, "16-bit Data 1:");
	t_u16_2 = new wxStaticText(this, -1, "16-bit Data 2:");
	t_u8_1 = new wxStaticText(this, -1, "8-bit Data 1:");
	t_u8_2 = new wxStaticText(this, -1, "8-bit Data 2:");
	t_u8_3 = new wxStaticText(this, -1, "8-bit Data 3:");
	t_u8_4 = new wxStaticText(this, -1, "8-bit Data 4:");

	sizer->Add(t_u32_1, 0, wxALIGN_CENTRE);
	sizer->Add(u32_1, 0, wxALIGN_CENTRE);
	sizer->Add(t_u16_1, 0, wxALIGN_CENTRE);
	sizer->Add(u16_1, 0, wxALIGN_CENTRE);
	sizer->Add(t_u16_2, 0, wxALIGN_CENTRE);
	sizer->Add(u16_2, 0, wxALIGN_CENTRE);
	sizer->Add(t_u8_1, 0, wxALIGN_CENTRE);
	sizer->Add(u8_1, 0, wxALIGN_CENTRE);
	sizer->Add(t_u8_2, 0, wxALIGN_CENTRE);
	sizer->Add(u8_2, 0, wxALIGN_CENTRE);
	sizer->Add(t_u8_3, 0, wxALIGN_CENTRE);
	sizer->Add(u8_3, 0, wxALIGN_CENTRE);
	sizer->Add(t_u8_4, 0, wxALIGN_CENTRE);
	sizer->Add(u8_4, 0, wxALIGN_CENTRE);
	sizer->Add(ok_button, 0, wxALIGN_CENTRE);
	sizer->Add(cancel_button, 0, wxALIGN_CENTRE);

	SetSizer(sizer);
	SetAutoLayout(TRUE);
	Layout();
}

ccTileDialog::~ccTileDialog()	{
	delete cancel_button;
	delete ok_button;
	delete u32_1;
	delete u16_1;
	delete u16_2;
	delete u8_1;
	delete u8_2;
	delete u8_3;
	delete u8_4;
	delete t_u32_1;
	delete t_u16_1;
	delete t_u16_2;
	delete t_u8_1;
	delete t_u8_2;
	delete t_u8_3;
	delete t_u8_4;
}

void ccTileDialog::updateTile()	{
	tile->set32bitUserData(strtoul(u32_1->GetLineText(0).c_str(), NULL, 0));
	tile->set16bitUserData(1, strtoul(u16_1->GetLineText(0).c_str(), NULL, 0));
	tile->set16bitUserData(2, strtoul(u16_2->GetLineText(0).c_str(), NULL, 0));
	tile->set8bitUserData(1, strtoul(u8_1->GetLineText(0).c_str(), NULL, 0));
	tile->set8bitUserData(2, strtoul(u8_2->GetLineText(0).c_str(), NULL, 0));
	tile->set8bitUserData(3, strtoul(u8_3->GetLineText(0).c_str(), NULL, 0));
	tile->set8bitUserData(4, strtoul(u8_4->GetLineText(0).c_str(), NULL, 0));
}
